function CytoSpectreGUI

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Set this to true before deploying in order to enable standalone version features.
standalone = 0;

%-------------------------INITIALIZATION------------------------------%
try
    % Tooltip texts
    tt.usedsettings = 'Settings used for analyzing the selected image.';
    fibstatline1 = 'Mean orientation: dominant orientation, ccw from positive x-axis (0-180 degrees).';
    fibstatline2 = 'Circular variance: a measure of the spread of the orientation distribution';
    fibstatline3 = '(0-1, 0=perfect isotropy, 1=perfect anisotropy).';
    fibstatline4 = 'Angular standard deviation: an alternative measure of the spread of the orientation distribution';
    fibstatline5 = '(0-sqrt(2), 0=perfect isotropy, sqrt(2)=perfect anisotropy).';
    fibstatline6 = 'Circular skewness: a measure of the symmetry of the orientation distribution.';
    fibstatline7 = 'Values close to 0 are indicative of a distribution that is symmetric around the mean orientation.';
    fibstatline8 = 'Circular kurtosis: a measure of the peakedness of the orientation distribution.';
    fibstatline9 = 'Low values indicate a smooth distribution, while high values indicate a spiky distribution.';
    fibstatline10 = 'Wavelength mean: mean of the wavelength distribution.';
    fibstatline11 = 'Wavelength median: median of the wavelength distribution.';
    fibstatline12 = 'Wavelength mode: mode of the wavelength distribution.';
    fibstatline13 = 'Wavelength standard deviation: standard deviation of the wavelength distribution.';
    tt.fibstats = sprintf([fibstatline1,'\n',' ','\n',fibstatline2,'\n',fibstatline3,'\n',' ','\n',fibstatline4,'\n',fibstatline5,'\n',' ','\n',fibstatline6,'\n',fibstatline7,'\n',' ','\n',fibstatline8,'\n',fibstatline9,'\n',' ','\n',fibstatline10,'\n',' ','\n',fibstatline11,'\n',' ','\n',fibstatline12,'\n',' ','\n',fibstatline13,'\n',' ']);
    cellstatline1 = 'Center X/Y: coordinates of the cell''s centroid starting from the top left corner.';
    cellstatline2 = 'Area: the area covered by the cell.';
    cellstatline3 = 'Perimeter: the perimeter of the cell i.e. the distance along the outline of the cell.';
    cellstatline4 = 'Major axis: the length of the cell along its longitudinal axis.';
    cellstatline5 = 'Minor axis: the length of the cell along its transverse axis.';
    cellstatline6 = 'Eccentricity: a measure of the "circularity" of the cell''s shape (0-1, 0=perfect circle, 1=line segment).';
    cellstatline7 = 'Solidity: a measure of the "smoothness" of the cell''s shape (0-1, smooth cells have a high value, cells with protrusions have a low value.';
    tt.cellstats = sprintf([cellstatline1,'\n',cellstatline2,'\n',cellstatline3,'\n',cellstatline4,'\n',cellstatline5,'\n',cellstatline6,'\n',cellstatline7]);
    tt.mixedbutton = 'Represents the complete spectrum within the wavelength limits i.e. objects of all sizes.';
    tt.detailbutton = 'Represents detailed structures within a certain wavelength range i.e. objects of certain characteristic size.';
    tt.quality = 'A measure of detail component signal strength relative to spectral background.';
    tt.celllist = 'Select cell (if segmentation enabled).';

    % Initialize global variables.
    selectedfile = [];
    selectedcell = [];
    selectedcomponent = 1;
    previouspath = [];
    acceptedtypes = {'*.tif;*.TIF;*.tiff;*.TIFF;*.jpg;*.jpeg;*.JPG;*.JPEG;*.gif;*.GIF;*.bmp;*.BMP;*.png;*.PNG',  'All images'; ...
                    '*.jpg;*.jpeg;*.JPG;*.JPEG',  'JPEG'; ...
                    '*.tif;*.TIF;*.tiff;*.TIFF',  'TIF'; ... 
                    '*.gif;*.GIF;',               'GIF'; ...
                    '*.bmp;*.BMP;',               'BMP'; ...
                    '*.png;*.PNG;',               'PNG'; ...
                    '*.*',                        'All Files (*.*)'};

    % Initialize input and output structures.
    input.fullfilenames = {};
    input.shortnames = {};
    input.inputimage = {};
    input.processed = [];
    result = struct('cellregion',[],'cellids',1,'celldata',[],'fibrildata',[],'spectra',[],'analysisparameters',[],'analysissettings',[]);

    % Initialize default general settings.
    generalsettings.temppath = 'sameasimages';
    % Correction factor for converting all units from um to other units.
    generalsettings.dimensionfactor = 1;
    % The symbol corresponding to the length units used.
    generalsettings.unitsymbol = [char(0181),'m'];

    % Initialize default analysis settings.
    % Detail extraction default settings, used for resetting.
    analysissettings.default_segmentlength = 0.1; % Proportion of total data length.
    analysissettings.default_alpha = 0.05;
    analysissettings.default_angleslack = 10; % In degrees.
    analysissettings.default_convergence_threshold = 1; % In degrees.
    analysissettings.default_maxiterations = 10;
    analysissettings.default_sigma_prior_angle = 0.5;

    % Color channel and camera.
    analysissettings.issegmentationon = 0;
    analysissettings.targetchannel = 4;
    % Camera.
    analysissettings.ismagnificationauto = 0;
    analysissettings.magnification = 40;
    analysissettings.camerapixelsize = 6.8; % In micrometers.
    analysissettings.imagepixelsize = generalsettings.dimensionfactor*analysissettings.camerapixelsize/analysissettings.magnification;
    analysissettings.resolution = 1/analysissettings.imagepixelsize;
    % Spectral.
    analysissettings.resolutionparameter = 3;
    % General wavelength range.
    analysissettings.islowlimitauto = 1;
    analysissettings.ishighlimitauto = 1;
    analysissettings.isexcludeon = 0;
    analysissettings.lowlimit = [];
    analysissettings.highlimit = [];
    analysissettings.excludelow = [];
    analysissettings.excludehigh = [];
    % Detail wavelength.
    analysissettings.isdetailauto = 1;
    analysissettings.highguess = [];
    analysissettings.lowguess = [];
    % Orientation correction.
    analysissettings.rotatemixed = 1;
    analysissettings.rotatedetail = 1;
    % Other detail extraction settings.
    analysissettings.isdetaildefault = 1;
    analysissettings.segmentlength = analysissettings.default_segmentlength;
    analysissettings.alpha = analysissettings.default_alpha;
    analysissettings.angleslack = analysissettings.default_angleslack;
    analysissettings.convergence_threshold = analysissettings.default_convergence_threshold;
    analysissettings.maxiterations = analysissettings.default_maxiterations;
    analysissettings.sigma_prior_angle = analysissettings.default_sigma_prior_angle;

    % Initialize main figure.
    screensize = get(0,'ScreenSize');
    defaultBackground = get(0,'defaultUicontrolBackgroundColor');
    mainfigure = figure('Name','CytoSpectre 1.0','NumberTitle','Off','MenuBar','None','Toolbar','Figure','Color',defaultBackground,'CloseRequestFcn',@closeProgram,'Visible','off','Resize','On','units','pixels','OuterPosition',[1 0.05*screensize(4) screensize(3) 0.95*screensize(4)]);
    toolbarhandle = findall(mainfigure,'tag','FigureToolBar');
    toolbarhandles = findall(toolbarhandle);
    try
        delete(toolbarhandles([2;3;4;5;6;7;9;13;14;15;16;17;18]));
    catch
    end

    % Initialize toolbar.
    toolbarfile = uimenu(mainfigure,'Label','File');
    toolbarsaveprofile = uimenu(toolbarfile,'Label','Save profile','Callback',@toolbarsaveprofile_Callback);
    toolbarloadprofile = uimenu(toolbarfile,'Label','Load profile','Callback',@toolbarloadprofile_Callback);
    toolbarquit = uimenu(toolbarfile,'Label','Exit','Callback',@toolbarquit_Callback);
    toolbarprefs1 = uimenu(mainfigure,'Label','Preferences');
    toolbarprefs2 = uimenu(toolbarprefs1,'Label','Analysis settings','Callback',@toolbarprefs_Callback);
    toolbarprefs3 = uimenu(toolbarprefs1,'Label','General settings','Callback',@toolbargeneral_Callback);
    toolbarabout1 = uimenu(mainfigure,'Label','About');
    toolbarabout2 = uimenu(toolbarabout1,'Label','About CytoSpectre','Callback',@toolbarabout_Callback);

    % Initialize listbox and its buttons.
    imagelist = uicontrol(mainfigure,'Style','listbox','Min',1,'Max',1,'String',input.shortnames,'Callback',@imagelist_Callback,'units','normalized','position',[0.01 0.1 0.15 0.89],'BackgroundColor',[1 1 1]);
    imageadd = uicontrol(mainfigure,'Style','push','String','Add','Callback',@imageadd_Callback,'units','normalized','position',[0.01 0.05 0.075 0.04]);
    imageremove = uicontrol(mainfigure,'Style','push','String','Remove','Callback',@imageremove_Callback,'units','normalized','position',[0.085 0.05 0.075 0.04]);
    imageclear = uicontrol(mainfigure,'Style','push','String','Clear all','Callback',@imageclear_Callback,'units','normalized','position',[0.01 0.01 0.15 0.04]);

    % Initialize main controls.
    analyzeall = uicontrol(mainfigure,'Style','push','String','Analyze all images','Callback',@analyzeall_Callback,'units','normalized','position',[0.84 0.01 0.15 0.08]);
    analyzethis = uicontrol(mainfigure,'Style','push','String','Analyze this image','Callback',@analyzethis_Callback,'units','normalized','position',[0.68 0.01 0.15 0.08]);
    exportresults = uicontrol(mainfigure,'Style','push','String','Export results','Callback',@exportresults_Callback,'units','normalized','position',[0.52 0.01 0.15 0.08]);

    % Initialize input image panel.
    imagepanel = uipanel(mainfigure,'units','normalized','position',[0.17 0.57 0.320 0.42]);
    imagebox = axes('Parent',imagepanel,'Visible','Off','XTick',[],'YTick',[],'units','normalized','position',[0.02 0.02 0.96 0.96]);

    % Initialize image data panel.
    imagedatapanel = uipanel(mainfigure,'units','normalized','position',[0.5 0.79 0.490 0.20]);
    imagenamebox = uicontrol(imagedatapanel,'Style','text','String','Filename: ','units','normalized','position',[0.01 0.85 0.98 0.1],'HorizontalAlignment','left');
    imagedatabox1 = uicontrol(imagedatapanel,'Style','text','String','1 ','units','normalized','position',[0.01 0.05 0.32 0.7],'HorizontalAlignment','left','TooltipString',tt.usedsettings);
    imagedatabox2 = uicontrol(imagedatapanel,'Style','text','String','2 ','units','normalized','position',[0.34 0.05 0.32 0.7],'HorizontalAlignment','left','TooltipString',tt.usedsettings);
    imagedatabox3 = uicontrol(imagedatapanel,'Style','text','String','3 ','units','normalized','position',[0.67 0.05 0.32 0.7],'HorizontalAlignment','left','TooltipString',tt.usedsettings);
    displayImagedata(imagedatabox1,[],[],generalsettings.unitsymbol,'basic');
    displayImagedata(imagedatabox2,[],[],generalsettings.unitsymbol,'settings1');
    displayImagedata(imagedatabox3,[],[],generalsettings.unitsymbol,'settings2');

    % Initialize cell data panel.
    celldatapanel = uipanel(mainfigure,'units','normalized','position',[0.5 0.57 0.490 0.20]);
    celllist = uicontrol(celldatapanel,'Style','listbox','String',result(1).cellids,'Callback',@celllist_Callback,'units','normalized','position',[0.03 0.05 0.1 0.9],'BackgroundColor',[1 1 1],'TooltipString',tt.celllist);
    colnames = {'Center X (pixels)','Center Y (pixels)',['Center X (',generalsettings.unitsymbol,')'],['Center Y (',generalsettings.unitsymbol,')'], ...
                ['Area (',generalsettings.unitsymbol,'^2)'],['Perimeter (',generalsettings.unitsymbol,')'], ...
                ['Major axis (',generalsettings.unitsymbol,')'],['Minor axis (',generalsettings.unitsymbol,')'],'Eccentricity','Solidity'};
    cellnames = {'1'};
    celltable = uitable(celldatapanel,'ColumnName',colnames,'RowName',cellnames,'ColumnWidth','auto','units','normalized','position',[0.15 0.05 0.82 0.9],'BackgroundColor',[1 1 1],'TooltipString',tt.cellstats);

    % Initialize fibril data panel.
    fibrildatapanel = uipanel(mainfigure,'units','normalized','position',[0.17 0.1 0.82 0.45]);
    componentbox = uibuttongroup('Parent',fibrildatapanel,'Title','Select component:','SelectionChangeFcn',@component_Callback,'units','normalized','position',[0.01 0.48 0.16 0.5]);
    component1 = uicontrol(componentbox,'Style','radiobutton','String','Mixed component','units','normalized','position',[0.05 0.8 0.95 0.1],'TooltipString',tt.mixedbutton);
    component2 = uicontrol(componentbox,'Style','radiobutton','String','Detail component','units','normalized','position',[0.05 0.6 0.95 0.1],'TooltipString',tt.detailbutton);
    qualitybox = uicontrol(componentbox,'Style','text','String','Detail component SNR: ','units','normalized','position',[0.05 0.05 0.95 0.25],'HorizontalAlignment','left','TooltipString',tt.quality);
    plotbox1 = axes('Parent',fibrildatapanel,'Box','on','XTick',[],'YTick',[],'units','normalized','position',[0.21 0.14 0.23 0.72]);
    plotbox2 = axes('Parent',fibrildatapanel,'Box','on','XTick',[],'YTick',[],'units','normalized','position',[0.48 0.14 0.23 0.72]);
    plotOrientationPolar(plotbox1,[]);
    plotWavelengthDistribution(plotbox2,[],generalsettings.unitsymbol);
    fibstatbox = uicontrol(fibrildatapanel,'Style','text','String',' ','units','normalized','position',[0.75 0.02 0.23 0.85],'HorizontalAlignment','left','TooltipString',tt.fibstats);
    displayFibrilStats(fibstatbox,[],[]);
    viewspectrum = uicontrol(fibrildatapanel,'Style','push','String','View power spectrum','Callback',@viewspectrum_Callback,'units','normalized','position',[0.01 0.36 0.16 0.1]);

    % Show GUI.
    set(mainfigure,'Visible','on');

catch err
    if standalone
        errorbox = errordlg({'An error occurred while initializing the user interface. The program will terminate.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: initialize user interface','modal');
        uiwait(errorbox);
        try
            delete(gcf);
        catch
        end
        quit;
    else
        errorbox = errordlg({['An error occurred while initializing the user interface. The program will terminate. ', ...
                            'The cause could be a MATLAB compatibility issue. CytoSpectre was designed and tested with MATLAB R2014a. ', ...
                            'Please consider installing the standalone Windows application.']; ''; ...
                            'Error identifier: '; err.identifier}, ...
                            'Error: initialize user interface','modal');
        uiwait(errorbox);
        try
            delete(gcf);
        catch
        end
        return;
    end
end

%-------------------------UPDATE GUI----------------------------------%
function updatehighGUI()
    try
        % Get the selected file.
        newselectedfile = get(imagelist,'value');
        if ~isempty(newselectedfile) && ~isnan(newselectedfile) && isscalar(newselectedfile)
           selectedfile = newselectedfile;
        else
           selectedfile = 1;
           set(imagelist,'value',1);
        end
        % There are images.
        if ~isempty(input.fullfilenames)
           input.inputimage = imread(input.fullfilenames{selectedfile});
           cla(imagebox);
           imshow(input.inputimage,[],'Parent',imagebox);
           set(imagenamebox,'String',['Filename: ',input.fullfilenames{selectedfile}]);
           % Temporary files are saved in the image directories.
           if strcmpi(generalsettings.temppath,'sameasimages') && exist([input.fullfilenames{selectedfile},'.temp.MAT'],'file')
               temp = load([input.fullfilenames{selectedfile},'.temp.MAT']);
               result = temp.result;
               clear temp;
               set(celllist,'String',result(1).cellids,'value',1);
           % Temporary files are saved in a separate temp directory.
           elseif ~strcmpi(generalsettings.temppath,'sameasimages') && exist([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{selectedfile},filesep,'_'),':','_'),'.temp.MAT'],'file')
               temp = load([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{selectedfile},filesep,'_'),':','_'),'.temp.MAT']);
               result = temp.result;
               clear temp;
               set(celllist,'String',result(1).cellids,'value',1);
           % Temporary files do not exist for this image.
           else
               result = struct('cellregion',[],'cellids',1,'celldata',[],'fibrildata',[],'spectra',[],'analysisparameters',[],'analysissettings',[]);
               set(celllist,'String',result(1).cellids,'value',1);
           end
        % There are no images.
        elseif isempty(input.fullfilenames)
           result = struct('cellregion',[],'cellids',1,'celldata',[],'fibrildata',[],'spectra',[],'analysisparameters',[],'analysissettings',[]);
           cla(imagebox);
           set(celllist,'String',result(1).cellids,'value',1);
           set(imagenamebox,'String','Filename: ');
        end
    catch err
        if standalone
            errorbox = errordlg({'An error occurred while updating the user interface. The program will terminate. You may have to delete temporary files manually.'; ''; 'Error identifier: '; err.identifier}, ...
                                'Error: update user interface','modal');
            uiwait(errorbox);
            try
                delete(gcf);
            catch
            end
            quit;
        else
            errorbox = errordlg({['An error occurred while updating the user interface. The program will terminate. You may have to delete temporary files manually. ', ...
                                'The cause could be a MATLAB compatibility issue. CytoSpectre was designed and tested with MATLAB R2014a. ', ...
                                'Please consider installing the standalone Windows application.']; ''; ...
                                'Error identifier: '; err.identifier}, ...
                                'Error: update user interface','modal');
            uiwait(errorbox);
            try
                delete(gcf);
            catch
            end
        end
    end
    updatelowGUI;
end

function updatelowGUI()
    try
        % Get selected cell.
        newselectedcell = get(celllist,'value');
        if ~isempty(newselectedcell) && ~isnan(newselectedcell) && isscalar(newselectedcell)
           selectedcell = newselectedcell;
        else
           selectedcell = 1;
           set(celllist,'value',1);
        end
        % Update cell borders on image.
        if ~isempty(result(1).cellregion)
            plotCellBorder(imagebox,input.inputimage,result(1).cellregion,selectedcell);
        end
        % Update morphological stats.
        if ~isempty(result(1).celldata)
            % Stats by cell.
            cellnames = {uint16(result(1).cellids)};
            set(celltable,'RowName',cellnames);
            datatoshow = struct2cell(result(1).celldata')';
            set(celltable,'Data',datatoshow);
        elseif isempty(result(1).celldata) || isempty(input.fullfilenames)
            set(celltable,'Data',{});
            set(celltable,'RowName',{'1'});
        end

        if length(result) < selectedcell
            set(qualitybox,'String','Detail component SNR: ');
            displayImagedata(imagedatabox1,[],[],generalsettings.unitsymbol,'basic');
            displayImagedata(imagedatabox2,[],[],generalsettings.unitsymbol,'settings1');
            displayImagedata(imagedatabox3,[],[],generalsettings.unitsymbol,'settings2');
            plotOrientationPolar(plotbox1,[]);
            plotWavelengthDistribution(plotbox2,[],generalsettings.unitsymbol);
            displayFibrilStats(fibstatbox,[],[]);
        elseif isempty(result(selectedcell).fibrildata) || isempty(input.fullfilenames)
            set(qualitybox,'String','Detail component SNR: ');
            displayImagedata(imagedatabox1,[],[],generalsettings.unitsymbol,'basic');
            displayImagedata(imagedatabox2,[],[],generalsettings.unitsymbol,'settings1');
            displayImagedata(imagedatabox3,[],[],generalsettings.unitsymbol,'settings2');
            plotOrientationPolar(plotbox1,[]);
            plotWavelengthDistribution(plotbox2,[],generalsettings.unitsymbol);
            displayFibrilStats(fibstatbox,[],[]);
        else
            displayImagedata(imagedatabox1,result(1).analysissettings,result(selectedcell).analysisparameters,generalsettings.unitsymbol,'basic');
            displayImagedata(imagedatabox2,result(1).analysissettings,result(selectedcell).analysisparameters,generalsettings.unitsymbol,'settings1');
            displayImagedata(imagedatabox3,result(1).analysissettings,result(selectedcell).analysisparameters,generalsettings.unitsymbol,'settings2');
            set(qualitybox,'String',{'Detail component SNR: '; [num2str(result(selectedcell).fibrildata.detailquality),' dB']});
            if selectedcomponent == 1
                plotOrientationPolar(plotbox1,result(selectedcell).fibrildata.orientation_mixed);
                title(plotbox1,'Orientation, mixed component');
                plotWavelengthDistribution(plotbox2,result(selectedcell).fibrildata.distance_mixed, generalsettings.unitsymbol);
                title(plotbox2,'Wavelength, mixed component');
                displayFibrilStats(fibstatbox, result(selectedcell).fibrildata.orientation_mixed, result(selectedcell).fibrildata.distance_mixed, generalsettings.unitsymbol);
            elseif selectedcomponent == 2
                plotOrientationPolar(plotbox1,result(selectedcell).fibrildata.orientation_detail);
                title(plotbox1,'Orientation, detail component');
                plotWavelengthDistribution(plotbox2,result(selectedcell).fibrildata.distance_detail, generalsettings.unitsymbol);
                title(plotbox2,'Wavelength, detail component');
                displayFibrilStats(fibstatbox, result(selectedcell).fibrildata.orientation_detail, result(selectedcell).fibrildata.distance_detail, generalsettings.unitsymbol);
            end
        end
    catch err
        if standalone
            errorbox = errordlg({'An error occurred while updating the user interface. The program will terminate. You may have to delete temporary files manually.'; ''; 'Error identifier: '; err.identifier}, ...
                                'Error: update user interface','modal');
            uiwait(errorbox);
            try
                delete(gcf);
            catch
            end
            quit;
        else
            errorbox = errordlg({['An error occurred while updating the user interface. The program will terminate. You may have to delete temporary files manually. ', ...
                                'The cause could be a MATLAB compatibility issue. CytoSpectre was designed and tested with MATLAB R2014a. ', ...
                                'Please consider installing the standalone Windows application.']; ''; ...
                                'Error identifier: '; err.identifier}, ...
                                'Error: update user interface','modal');
            uiwait(errorbox);
            try
                delete(gcf);
            catch
            end
        end
    end
end

%-----------------TOOLBAR AND GENERAL----------------------------------%
% FILE -> SAVE PROFILE.
function toolbarsaveprofile_Callback(hObject,eventdata)
    try
        [filename,pathname] = uiputfile('*.mat','Save profile as','defaultprofile.mat');
        if filename == 0
            return;
        else
            outputfile = [pathname,filename];
        end
        save(outputfile,'analysissettings','generalsettings');
        waitdialog = dialog('units','pixels','OuterPosition',[0.3*screensize(3) 0.4*screensize(4) 0.4*screensize(3) 0.2*screensize(4)], ...
            'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
        waittext = uicontrol(waitdialog,'Style','text','String','Profile saved','units','normalized','position',[0.1 0.4 0.8 0.5],'HorizontalAlignment','center');
        pause(0.5);
        set(waitdialog,'CloseRequestFcn','closereq');
        close(waitdialog);
    catch err
        if exist('waitdialog','var')
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        end
        errorbox = errordlg({'An error occurred while saving a profile.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: save profile','modal');
        uiwait(errorbox);
    end
end    

% FILE -> LOAD PROFILE.
function toolbarloadprofile_Callback(hObject,eventdata)
    try
        [filename, pathname] = uigetfile({'*.mat','*.MAT'},'Select profile to load','MultiSelect','off');
        if pathname == 0
            return;
        else
            inputfile = [pathname,filename];
        end
        varsinfile = whos('-file',inputfile);
        if ismember('analysissettings',{varsinfile.name}) && ismember('generalsettings',{varsinfile.name})
            [analysissettings, generalsettings] = loadProfile(inputfile);
            waitdialog = dialog('units','pixels','OuterPosition',[0.3*screensize(3) 0.4*screensize(4) 0.4*screensize(3) 0.2*screensize(4)], ...
                'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
            waittext = uicontrol(waitdialog,'Style','text','String','Profile loaded','units','normalized','position',[0.1 0.4 0.8 0.5],'HorizontalAlignment','center');
            pause(0.5);
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        else
            errorbox = errordlg('The selected file does not contain a valid profile!','Invalid file','modal');
            uiwait(errorbox);
            return;
        end
    catch err
        if exist('waitdialog','var')
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        end
        errorbox = errordlg({'An error occurred while loading a profile.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: load profile','modal');
        uiwait(errorbox);
    end
end

% FILE -> QUIT.
function toolbarquit_Callback(hObject,eventdata)
    closeProgram;
end

function closeProgram(hObject,eventdata)
    try
        quitbox = questdlg('Are you sure you want to quit? All unsaved data will be lost.','Quit','Yes','No','No');
        if strcmpi(quitbox,'Yes')
            % Remove temporary files.
            % Temp files in image directories.
            if strcmpi(generalsettings.temppath,'sameasimages')
                for filetodel=1:length(input.fullfilenames)
                    if exist([input.fullfilenames{filetodel},'.temp.MAT'],'file')
                        delete([input.fullfilenames{filetodel},'.temp.MAT']);
                    end
                end
            % Temp files in a separate temp directory.
            else
                for filetodel=1:length(input.fullfilenames)
                    if exist([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{filetodel},filesep,'_'),':','_'),'.temp.MAT'],'file')
                        delete([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{filetodel},filesep,'_'),':','_'),'.temp.MAT']);
                    end
                end
            end
            % Terminate program
            delete(gcf);
            if standalone
                quit;
            end
        else
            return;
        end
    catch err
        errorbox = errordlg({'An error occurred while closing the program. The program will now terminate. You may have to delete temporary files manually.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: close program','modal');
        uiwait(errorbox);
        % Terminate program.
        try
            delete(gcf);
        catch
        end
        if standalone
            quit;
        end
    end
end

% PREFERENCES.
function toolbarprefs_Callback(hObject,eventdata)
    try
        analysissettings = analysissettingsPrompt(analysissettings, generalsettings);
    catch err
        errorbox = errordlg({'An error occurred while adjusting analysis settings.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: analysis settings','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
end

function toolbargeneral_Callback(hObject,eventdata)
    try
        newgeneralsettings = generalsettingsPrompt(generalsettings);
        % The user is attempting to change the general settings in the middle
        % of an analysis session. Session must be cleared before changing.
        if (~strcmpi(newgeneralsettings.temppath,generalsettings.temppath) || newgeneralsettings.dimensionfactor ~= generalsettings.dimensionfactor) && ~isempty(input.fullfilenames)
            newgeneralbox = questdlg('Changing the general settings will clear the current session. All unsaved data will be lost. Are you sure you want to change the settings?','Change general settings?','Yes','No','No');
            if strcmpi(newgeneralbox,'Yes')
                imageclear_Callback;
                generalsettings = newgeneralsettings;
            end
        % The user did not change the temp directory or the current session is blank.
        else
            generalsettings = newgeneralsettings;
        end
        % Update cell table header, image pixel size and resolution as the
        % user might have changed length unit settings.
        colnames = {'Cell','Center X (pixels)','Center Y (pixels)',['Center X (',generalsettings.unitsymbol,')'],['Center Y (',generalsettings.unitsymbol,')'], ...
                ['Area (',generalsettings.unitsymbol,'^2)'],['Perimeter (',generalsettings.unitsymbol,')'], ...
                ['Major axis (',generalsettings.unitsymbol,')'],['Minor axis (',generalsettings.unitsymbol,')'],'Eccentricity','Solidity'};
        set(celltable,'ColumnName',colnames);
        plotWavelengthDistribution(plotbox2,[], generalsettings.unitsymbol);
        analysissettings.imagepixelsize = generalsettings.dimensionfactor*analysissettings.camerapixelsize/analysissettings.magnification;
        analysissettings.resolution = 1/analysissettings.imagepixelsize;
        displayImagedata(imagedatabox1,[],[],generalsettings.unitsymbol,'basic');
        displayImagedata(imagedatabox2,[],[],generalsettings.unitsymbol,'settings1');
        displayImagedata(imagedatabox3,[],[],generalsettings.unitsymbol,'settings2');
    catch err
        errorbox = errordlg({'An error occurred while adjusting general settings.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: general settings','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
end

% ABOUT.
function toolbarabout_Callback(hObject,eventdata)
    try
        showAboutBox;
    catch err
        errorbox = errordlg({'An unknown error occurred while showing "About" box.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: about','modal');
        uiwait(errorbox);
    end
end

%-------------------------FILE BROWSER--------------------------------%
% SELECTION CHANGE.
function imagelist_Callback(hObject,eventdata)
    updatehighGUI;
end

% ADD IMAGE.
function imageadd_Callback(hObject,eventdata)
    try
        if isempty(previouspath)
            [filename, pathname] = uigetfile(acceptedtypes,'Select image to analyze','MultiSelect','on');
        else
            [filename, pathname] = uigetfile(acceptedtypes,'Select image to analyze',previouspath,'MultiSelect','on');
        end
        if pathname == 0
            return;
        end

        % Multiple files were selected.
        if iscell(filename)
            % Create waitbox.
            waitdialog = dialog('units','pixels','OuterPosition',[0.3*screensize(3) 0.4*screensize(4) 0.4*screensize(3) 0.2*screensize(4)], ...
                'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
            waittext = uicontrol(waitdialog,'Style','text','String','Loading images...','units','normalized','position',[0.1 0.4 0.8 0.5],'HorizontalAlignment','center');
            drawnow;
            for k=1:length(filename)
                try
                    addedimage = imread([pathname filename{k}]);
                    % Check that the image is grayscale or RGB.
                    if (size(addedimage,3) ~= 1) && (size(addedimage,3) ~= 3)
                        errorbox = errordlg(['The image ',[pathname filename{k}],' is not in grayscale or RGB format! The image was not loaded.'],'Error','modal');
                        uiwait(errorbox);
                        continue;
                    else
                        % Check that an image with identical name was not added.
                        if isempty(find(strcmp(input.fullfilenames,[pathname filename{k}]),1))
                            input.fullfilenames = [input.fullfilenames [pathname filename{k}]];
                            input.shortnames = [input.shortnames filename{k}];
                            input.processed = [input.processed 0];
                        else
                            errorbox = warndlg(['An image with the filename ',[pathname filename{k}],' was already added!'],'Warning','modal');
                            uiwait(errorbox);
                            continue;
                        end
                    end
                catch
                    errorbox = errordlg(['The image ',[pathname filename{k}],' could not be loaded! Check that the image is not corrupted.'],'Error','modal');
                    uiwait(errorbox);
                    continue;
                end
            end
            set(waitdialog,'CloseRequestFcn','closereq');
            pause(0.5);
            close(waitdialog);
        % A single file was selected.
        else
            try
                addedimage = imread([pathname filename]);
                % Check that the image is grayscale or RGB.
                if (size(addedimage,3) ~= 1) && (size(addedimage,3) ~= 3)
                    errorbox = errordlg(['The image ',[pathname filename],' is not in grayscale or RGB format! The image was not loaded.'],'Error','modal');
                    uiwait(errorbox);
                else
                    % Check that an image with identical name was not added.
                    if isempty(find(strcmp(input.fullfilenames,[pathname filename]),1))
                        input.fullfilenames = [input.fullfilenames [pathname filename]];
                        input.shortnames = [input.shortnames filename];
                        input.processed = [input.processed 0];
                    else
                        errorbox = warndlg(['An image with the filename ',[pathname filename],' was already added!'],'Warning','modal');
                        uiwait(errorbox);
                    end
                end
            catch
                errorbox = errordlg(['The image ',[pathname filename],' could not be loaded! Check that the image is not corrupted.'],'Error','modal');
                uiwait(errorbox);
            end
        end
        set(imagelist,'String',input.shortnames,'value',1);
        selectedfile = 1;
        previouspath = pathname;
    catch err
        if exist('waitdialog','var')
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        end
        errorbox = errordlg({'An error occurred while adding image.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: add','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
end

% REMOVE IMAGE.
function imageremove_Callback(hObject,eventdata)
    try
        selectedfile = get(imagelist,'value');
        if ~isempty(selectedfile) && ~isempty(input.fullfilenames)
            % Delete temporary file.
            if strcmpi(generalsettings.temppath,'sameasimages') && exist([input.fullfilenames{selectedfile},'.temp.MAT'],'file')
                delete([input.fullfilenames{selectedfile},'.temp.MAT']);
            elseif ~strcmpi(generalsettings.temppath,'sameasimages') && exist([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{selectedfile},filesep,'_'),':','_'),'.temp.MAT'],'file')
                delete([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{selectedfile},filesep,'_'),':','_'),'.temp.MAT']);
            end
            input.fullfilenames(selectedfile) = [];
            input.shortnames(selectedfile) = [];
            input.processed(selectedfile) = [];
            input.inputimage = [];
            set(imagelist,'String',input.shortnames);
            if isempty(input.fullfilenames)
                set(imagelist,'value',[]);
            end
            if selectedfile > numel(input.fullfilenames)
                set(imagelist,'value',numel(input.fullfilenames));
            end
        end
    catch err
        errorbox = errordlg({'An error occurred while removing image.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: remove','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
end

% CLEAR ALL IMAGES.
function imageclear_Callback(hObject,eventdata)
    try
        % Remove temporary files.
        % Temp files in image directories.
        if strcmpi(generalsettings.temppath,'sameasimages')
            for filetodel=1:length(input.fullfilenames)
                if exist([input.fullfilenames{filetodel},'.temp.MAT'],'file')
                    delete([input.fullfilenames{filetodel},'.temp.MAT']);
                end
            end
        % Temp files in a separate temp directory.
        else
            for filetodel=1:length(input.fullfilenames)
                if exist([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{filetodel},filesep,'_'),':','_'),'.temp.MAT'],'file')
                    delete([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{filetodel},filesep,'_'),':','_'),'.temp.MAT']);
                end
            end
        end
        input.fullfilenames = {};
        input.shortnames = {};
        input.inputimage = [];
        input.processed = [];
        set(imagelist,'String',input.shortnames,'value',[]);
    catch err
        errorbox = errordlg({'An unknown error occurred while clearing images.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: clear all','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
end

%-------------------------MAIN CONTROLS-------------------------------%
% ANALYZE ALL IMAGES.
function analyzeall_Callback(hObject,eventdata)
    try
        if ~isempty(input.fullfilenames)
            % Ask if the user wishes to analyze all images or only images
            % that have not been fully analyzed yet. If no images have been
            % analyzed yet, no need to ask.
            if all(input.processed == 0)
                allornew = 'Analyze all';
            else
                allornew = questdlg('Do you wish to (re)analyze all images or skip previously fully analyzed images? Reanalyzing will overwrite any existing results.','Start analysis run', ...
                                    'Analyze all','Skip analyzed images','Cancel','Cancel');
            end

            % User requested all images to be analyzed.
            if strcmp(allornew,'Analyze all')
                filestoanalyze = 1:length(input.fullfilenames);
            % User requested previously analyzed images to be skipped.
            elseif strcmp(allornew,'Skip analyzed images')
                filestoanalyze = find(input.processed == 0);
            % The user cancelled.
            else
                return;
            end

            % Reset abort flag.
            abortflag = 0;
            % Create waitbox.
            waitdialog = dialog('units','pixels','OuterPosition',[0.3*screensize(3) 0.4*screensize(4) 0.4*screensize(3) 0.2*screensize(4)], ...
                'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
            waittext = uicontrol(waitdialog,'Style','text','String','Starting analysis...','units','normalized','position',[0.1 0.4 0.8 0.5],'HorizontalAlignment','center');
            abortbutton = uicontrol(waitdialog,'Style','push','String','Abort','Callback',@abortrun_Callback,'units','normalized','position',[0.3 0.1 0.4 0.2]);
            drawnow;

            % Loop through all files.
            timeperim = 10;
            totalfilenum = length(filestoanalyze);
            for iter = 1:totalfilenum
                % Check if aborting the run has been requested.
                if abortflag == 1
                    break;
                end

                % Estimate required time.
                tic;
                averagetime = round(mean(timeperim));
                estimatedtimeleft = averagetime*(totalfilenum-iter+1);
                secsleft = mod(estimatedtimeleft,60);
                minsleft = (estimatedtimeleft - secsleft)/60;

                % Read image.
                thisfile = filestoanalyze(iter);
                input.inputimage = imread(input.fullfilenames{thisfile});

                % Detect magnification automatically from filename if enabled.
                if analysissettings.ismagnificationauto
                    detectedmagnification = detectMagnification(input.fullfilenames{thisfile});
                    if ~isempty(detectedmagnification)
                        analysissettings.magnification = detectedmagnification;
                        analysissettings.imagepixelsize = generalsettings.dimensionfactor*analysissettings.camerapixelsize/analysissettings.magnification;
                        analysissettings.resolution = 1/analysissettings.imagepixelsize;
                    end
                end

                % Reset the result structure.
                clear result;
                result = struct('cellregion',[],'cellids',1,'celldata',[],'fibrildata',[],'spectra',[],'analysisparameters',[],'analysissettings',[]);

                % Get segmentation if requested.
                if analysissettings.issegmentationon == 1
                    result(1).cellregion = getSegmentation(input.fullfilenames{thisfile});
                    % If no external mask was found, issue a warning.
                    if isempty(result(1).cellregion)
                        errorbox = warndlg({['No custom segmentation mask found for ',input.shortnames{thisfile},'! ']; '' ; ...
                                            ['Mask with segmented cells should be named segcell_',input.shortnames{thisfile},'. '] ; ...
                                            'Segmentation masks should be placed in the same directory as the image.'; ...
                                            'Mask images should be binary (i.e. black and white) with dark background and bright cells/nuclei.'}, ...
                                            'Warning: skipping segmentation','modal');
                        pause(2);
                        if ishandle(errorbox)
                            close(errorbox);
                        end
                    % If a mask was found but it has incorrect size, issue
                    % a warning and rescale the mask.
                    elseif ~isempty(result(1).cellregion) && any(size(result(1).cellregion) ~= size(input.inputimage(:,:,1)))
                        errorbox = warndlg({['The size of the custom cell segmentation mask for ',input.shortnames{selectedfile}, ' differs from the size of the image!']; '' ; ...
                                            ['The size of the mask image is ',num2str(size(result(1).cellregion,2)),' (width) x ',num2str(size(result(1).cellregion,1)),' (height) pixels.']; ...
                                            ['The mask image will be rescaled to ',num2str(size(input.inputimage,2)),' (width) x ',num2str(size(input.inputimage,1)),' (height) pixels.']}, ...
                                            'Warning: rescaling segmentation mask','modal');
                        result(1).cellregion = imresize(result(1).cellregion,[size(input.inputimage(:,:,1),1) size(input.inputimage(:,:,1),2)],'nearest');
                        pause(2);
                        if ishandle(errorbox)
                            close(errorbox);
                        end
                    end
                end

                % Analyze morphology if segmentation was performed, or set
                % morphological data to empty.
                if analysissettings.issegmentationon == 1 && ~isempty(result(1).cellregion)
                    result(1).cellids = 1:max(result(1).cellregion(:));
                    [result(1).celldata, ~] = analyzeMorphology(result(1).cellregion,analysissettings.imagepixelsize);
                end

                % Analyze cells and save results.
                totalcellnum = length(result(1).cellids);
                for cellind = 1:totalcellnum
                    set(waittext,'String',{['Analyzing image ',num2str(iter),'/',num2str(totalfilenum)]; ...
                                           ['Analyzing cell ',num2str(cellind),'/',num2str(totalcellnum)]; ...
                                           ['Estimated total time left: ',num2str(minsleft),' minutes, ',num2str(secsleft),' seconds...']});
                    drawnow;
                    thiscellregion = result(1).cellregion == cellind;
                    [result(cellind).fibrildata, result(cellind).spectra, result(cellind).analysisparameters] = mainAnalysis(input.inputimage, thiscellregion, analysissettings);
                end
                result(1).analysissettings = analysissettings;
                if strcmpi(generalsettings.temppath,'sameasimages')
                    save([input.fullfilenames{thisfile},'.temp.MAT'],'result');
                else
                    save([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{thisfile},filesep,'_'),':','_'),'.temp.MAT'],'result');
                end
                input.processed(thisfile) = 1;

                % Update time per image.
                timeperim(iter) = toc;
            end

            % Kill waitbox and reset abort flag if necessary.
            if abortflag == 0
                set(waittext,'String','Analysis run finished');
                pause(0.5);
                set(waitdialog,'CloseRequestFcn','closereq');
                close(waitdialog);
            elseif abortflag == 1
                abortflag = 0;
                set(waittext,'String','Analysis run aborted');
                pause(0.5);
                set(waitdialog,'CloseRequestFcn','closereq');
                close(waitdialog);
            end
        end
    catch err
        if exist('waitdialog','var')
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        end
        errorbox = errordlg({'An error occurred while analyzing images.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: analyze images','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
    % ABORT ANALYSIS RUN IF REQUESTED.
    function abortrun_Callback(hObject,eventdata)
        abortflag = 1; 
    end
end

% ANALYZE THIS IMAGE
function analyzethis_Callback(hObject,eventdata)
    try
        if ~isempty(input.fullfilenames)
            % Create waitbox.
            waitdialog = dialog('units','pixels','OuterPosition',[0.3*screensize(3) 0.4*screensize(4) 0.4*screensize(3) 0.2*screensize(4)], ...
                                'WindowStyle','modal','Visible','On','ButtonDownFcn','','CloseRequestFcn','');
            waittext = uicontrol(waitdialog,'Style','text','String','Starting analysis...','units','normalized','position',[0.1 0.4 0.8 0.5],'HorizontalAlignment','center');
            drawnow;

            input.inputimage = imread(input.fullfilenames{selectedfile});

            % Detect magnification automatically from filename if enabled.
            if analysissettings.ismagnificationauto
                detectedmagnification = detectMagnification(input.fullfilenames{selectedfile});
                if ~isempty(detectedmagnification)
                    analysissettings.magnification = detectedmagnification;
                    analysissettings.imagepixelsize = generalsettings.dimensionfactor*analysissettings.camerapixelsize/analysissettings.magnification;
                    analysissettings.resolution = 1/analysissettings.imagepixelsize;
                end
            end

            % Reset the result structure.
            clear result;
            result = struct('cellregion',[],'cellids',1,'celldata',[],'fibrildata',[],'spectra',[],'analysisparameters',[],'analysissettings',[]);

            % Get segmentation if requested.
            if analysissettings.issegmentationon == 1
                result(1).cellregion = getSegmentation(input.fullfilenames{selectedfile});
                % If no external mask was found, issue a warning.
                if isempty(result(1).cellregion)
                    errorbox = warndlg({['No custom segmentation mask found for ',input.shortnames{selectedfile},'! ']; '' ; ...
                                        ['Mask with segmented cells should be named segcell_',input.shortnames{selectedfile},'. '] ; ...
                                        'Segmentation masks should be placed in the same directory as the image.'; ...
                                        'Mask images should be binary (i.e. black and white) with dark background and bright cells/nuclei.'}, ...
                                        'Warning: skipping segmentation','modal');
                    uiwait(errorbox);
                % If a mask was found but it has incorrect size, issue
                % a warning and rescale the mask.
                elseif ~isempty(result(1).cellregion) && any(size(result(1).cellregion) ~= size(input.inputimage(:,:,1)))
                    errorbox = warndlg({['The size of the custom cell segmentation mask for ',input.shortnames{selectedfile}, ' differs from the size of the image!']; '' ; ...
                                        ['The size of the mask image is ',num2str(size(result(1).cellregion,2)),' (width) x ',num2str(size(result(1).cellregion,1)),' (height) pixels.']; ...
                                        ['The mask image will be rescaled to ',num2str(size(input.inputimage,2)),' (width) x ',num2str(size(input.inputimage,1)),' (height) pixels.']}, ...
                                        'Warning: rescaling segmentation mask','modal');
                    result(1).cellregion = imresize(result(1).cellregion,[size(input.inputimage(:,:,1),1) size(input.inputimage(:,:,1),2)],'bilinear');
                    uiwait(errorbox);
                end
            end

            % Analyze morphology if segmentation was performed.
            if analysissettings.issegmentationon == 1 && ~isempty(result(1).cellregion)
                result(1).cellids = 1:max(result(1).cellregion(:));
                [result(1).celldata, ~] = analyzeMorphology(result(1).cellregion,analysissettings.imagepixelsize);
            end

            % Analyze cells and save results.
            totalcellnum = length(result(1).cellids);
            for cellind = 1:totalcellnum
                set(waittext,'String',{['Analyzing ',input.shortnames{selectedfile}]; ...
                                       ['Analyzing cell ',num2str(cellind),'/',num2str(totalcellnum)]; ...
                                       'Please wait...'});
                drawnow;
                thiscellregion = result(1).cellregion == cellind;
                [result(cellind).fibrildata, result(cellind).spectra, result(cellind).analysisparameters] = mainAnalysis(input.inputimage, thiscellregion, analysissettings);
            end
            result(1).analysissettings = analysissettings;
            if strcmpi(generalsettings.temppath,'sameasimages')
                save([input.fullfilenames{selectedfile},'.temp.MAT'],'result');
            else
                save([generalsettings.temppath,filesep,strrep(strrep(input.fullfilenames{selectedfile},filesep,'_'),':','_'),'.temp.MAT'],'result');
            end
            input.processed(selectedfile) = 1;

            % Kill waitbox
            set(waittext,'String','Analysis finished');
            pause(0.5);
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        end
    catch err
        if exist('waitdialog','var')
            set(waitdialog,'CloseRequestFcn','closereq');
            close(waitdialog);
        end
        errorbox = errordlg({'An error occurred while analyzing image.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: analyze image','modal');
        uiwait(errorbox);
    end
    updatehighGUI;
end

% EXPORT RESULT.
function exportresults_Callback(hObject,eventdata)
    try
        if isempty(input.fullfilenames)
            return;
        end

        % Ask the user for export settings.
        [thisorall, exportfiletype] = exportPrompt;
        if isempty(thisorall)
            return;
        end

        % Export data.
        if strcmp(thisorall,'This')
            if strcmp(exportfiletype,'spreadsheet') || strcmp(exportfiletype,'text')
                exportok = exportResultsSummary(input.fullfilenames(selectedfile),input.shortnames(selectedfile),generalsettings.temppath,exportfiletype,generalsettings.unitsymbol);
            else
                exportok = exportResultsRaw(input.fullfilenames(selectedfile),input.shortnames(selectedfile),generalsettings.temppath,exportfiletype,generalsettings.unitsymbol);
            end
        elseif strcmp(thisorall,'All')
            if strcmp(exportfiletype,'spreadsheet') || strcmp(exportfiletype,'text')
                exportok = exportResultsSummary(input.fullfilenames,input.shortnames,generalsettings.temppath,exportfiletype,generalsettings.unitsymbol);
            else
                exportok = exportResultsRaw(input.fullfilenames,input.shortnames,generalsettings.temppath,exportfiletype,generalsettings.unitsymbol);
            end
        end

        % Check if export was successful.
        if exportok == 0
            errorbox = errordlg('Data export failed. Plese verify the file is not open in another program and the disk is not full or write protected.',...
                                'Error during data export','modal');
            uiwait(errorbox);
        end
    catch err
        errorbox = errordlg({'An error occurred while exporting results.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: export results','modal');
        uiwait(errorbox);
    end
end

%-------------------------CELL DATA PANEL-----------------------------%
% SELECTION CHANGE
function celllist_Callback(hObject,eventdata)
    updatelowGUI;
end

%-------------------------FIBRIL DATA PANEL-----------------------------%
% SELECTION CHANGE
function component_Callback(~,callbackdata)
    if callbackdata.NewValue == callbackdata.OldValue
        return;
    end
    if callbackdata.NewValue == component1
        selectedcomponent = 1;
    elseif callbackdata.NewValue == component2
        selectedcomponent = 2;
    end
    updatelowGUI;
end

% VIEW SPECTRUM.
function viewspectrum_Callback(hObject,eventdata)
    try
        if isempty(result(1).fibrildata)
            return;
        end
        if isempty(result(selectedcell).fibrildata)
            return;
        end
        spectralfigure = figure('Name','Power spectrum','NumberTitle','Off','MenuBar','None','Toolbar','Figure','Color',defaultBackground);
        set(spectralfigure,'Resize','On','units','pixels','OuterPosition',[0.15*screensize(3) 0.15*screensize(4) 0.7*screensize(3) 0.7*screensize(4)]); 
        spectraltoolbarhandle = findall(spectralfigure,'tag','FigureToolBar');
        spectraltoolbarhandles = findall(spectraltoolbarhandle);
        try
            delete(spectraltoolbarhandles([2;3;4;5;6;7;9;13;14;15;16;17;18]));
        catch
        end

        spectralplot = axes('Parent',spectralfigure,'Box','on','XTick',[],'YTick',[],'units','normalized','position',[0.15 0.15 0.7 0.7]);
        plotSpectralROI(spectralplot, result(selectedcell).spectra, generalsettings.unitsymbol);
        title(spectralplot,{'Power spectrum (dB) in polar coordinates'; ' '; ...
            'Mixed component (black)'; ['Detail component (red): ',num2str(result(selectedcell).analysisparameters.iter_detail),' iterations']});
    catch err
        errorbox = errordlg({'An error occurred while plotting the power spectrum.'; ''; 'Error identifier: '; err.identifier}, ...
                            'Error: view power spectrum','modal');
        uiwait(errorbox);
    end
end

end
