function [cellstats, summarystats] = analyzeMorphology(cellregions,pixelsize)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Get properties of the cell regions.
propstomeasure = {'Centroid','Area','Perimeter','MajorAxisLength','MinorAxisLength','Eccentricity','Solidity'};
stats = regionprops(cellregions,propstomeasure);
cellstats = struct('CentroidXpix',[],'CentroidYpix',[],'CentroidXphys',[],'CentroidYphys',[],'Area',[],'Perimeter',[], ...
                    'MajorAxisLength',[],'MinorAxisLength',[],'Eccentricity',[],'Solidity',[]);

for regionind = 1:length(stats)
    % Centroid in pixels.
    cellstats(regionind).CentroidXpix = stats(regionind).Centroid(1);
    cellstats(regionind).CentroidYpix = stats(regionind).Centroid(2);
    % Centroid in physical units.
    cellstats(regionind).CentroidXphys = pixelsize*stats(regionind).Centroid(1);
    cellstats(regionind).CentroidYphys = pixelsize*stats(regionind).Centroid(2);

    % Area, perimeter and axis lengths in physical units.
    pixelarea = pixelsize^2;
    cellstats(regionind).Area = pixelarea*stats(regionind).Area;
    cellstats(regionind).Perimeter = pixelsize*stats(regionind).Perimeter;
    cellstats(regionind).MajorAxisLength = pixelsize*stats(regionind).MajorAxisLength;
    cellstats(regionind).MinorAxisLength = pixelsize*stats(regionind).MinorAxisLength;

    % Eccentricity and solidity.
    cellstats(regionind).Eccentricity = stats(regionind).Eccentricity;
    cellstats(regionind).Solidity = stats(regionind).Solidity;
end

% Calculate summary statistics.
tempstats = cell2mat(struct2cell(cellstats));
tempstats = reshape(tempstats,length(fieldnames(cellstats)),length(stats));
statvalues = num2cell([mean(tempstats,2)'; median(tempstats,2)'; max(tempstats,[],2)'; min(tempstats,[],2)'; var(tempstats,0,2)'; std(tempstats,0,2)'; skewness(tempstats,0,2)'; kurtosis(tempstats,0,2)']);
summarystats = cell2struct(statvalues,fieldnames(cellstats),2);