function displayImagedata(plothandle, analysissettings, analysispar, unitsymbol, datatodisplay)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Basic info about image and pixel size.
if strcmpi(datatodisplay,'basic')
    if isempty(analysissettings) || isempty(analysispar)
        set(plothandle,'String', ...
        {'Image size (pixels): '; ...
        ['Image size (',unitsymbol,'): ']; ...
        'Magnification: '; ...
        ['Camera pixel size (',char(0181),'m):']; ...
        ['Image pixel size (',unitsymbol,'):']; ...
        'Segmentation: '});
    else
        switch analysissettings.issegmentationon
            case 0
                segstring = 'Disabled';
            case 1
                segstring = 'Enabled';
        end
        set(plothandle,'String', ...
        {['Image size (pixels): ',num2str(analysispar.imsizex),' x ',num2str(analysispar.imsizey)]; ...
        ['Image size (',unitsymbol,'): ',num2str(analysispar.imsizex*analysissettings.imagepixelsize),' x ',num2str(analysispar.imsizey*analysissettings.imagepixelsize)]; ...
        ['Magnification: ',num2str(analysissettings.magnification)]; ...
        ['Camera pixel size (',char(0181),'m): ',num2str(analysissettings.camerapixelsize)]; ...
        ['Image pixel size (',unitsymbol,'): ',num2str(analysissettings.imagepixelsize)]; ...
        ['Segmentation: ',segstring]});
    end
% Analysis settings.
elseif strcmpi(datatodisplay,'settings1')
    if isempty(analysissettings) || isempty(analysispar)
        set(plothandle,'String', ...
        {'Target channel: '; ...
        'Resolution/noise: '; ...
        'Rotate mixed: '; ...
        'Rotate detail: '; ...
        ['Min wavelength (',unitsymbol,'): ']; ...
        ['Max wavelength (',unitsymbol,'): ']});
    else
        switch analysissettings.targetchannel
            case 1
                chstring = 'Red';
            case 2
                chstring = 'Green';
            case 3
                chstring = 'Blue';
            case 4
                chstring = 'Combined';
        end
        switch analysissettings.resolutionparameter
            case 1
                resostring = 'Min noise';
            case 2
                resostring = 'Low noise';
            case 3
                resostring = 'Balanced';
            case 4
                resostring = 'High resolution';
            case 5
                resostring = 'Max resolution';
        end
        switch analysissettings.rotatemixed
            case 0
                rotmixstring = 'Disabled';
            case 1
                rotmixstring = 'Enabled';
        end
        switch analysissettings.rotatedetail
            case 0
                rotdetstring = 'Disabled';
            case 1
                rotdetstring = 'Enabled';
        end
        switch analysissettings.islowlimitauto
            case 0
                wlminstring = num2str(analysissettings.lowlimit);
            case 1
                wlminstring = 'Auto';
        end
        switch analysissettings.ishighlimitauto
            case 0
                wlmaxstring = num2str(analysissettings.highlimit);
            case 1
                wlmaxstring = 'Auto';
        end
        set(plothandle,'String', ...
        {['Target channel: ',chstring]; ...
        ['Resolution/noise: ',resostring]; ...
        ['Rotate mixed: ',rotmixstring]; ...
        ['Rotate detail: ',rotdetstring]; ...
        ['Min wavelength (',unitsymbol,'): ',num2str(1/analysispar.highestfreq),' / ',wlminstring]; ...
        ['Max wavelength (',unitsymbol,'): ',num2str(1/analysispar.lowestfreq),' / ',wlmaxstring]});
    end
% More analysis settings.
elseif strcmpi(datatodisplay,'settings2')
    if isempty(analysissettings) || isempty(analysispar)
        set(plothandle,'String', ...
        {['Excluded, min (',unitsymbol,'): ']; ...
        ['Excluded, max (',unitsymbol,'): ']; ...
        ['Detail guess, min (',unitsymbol,'): ']; ...
        ['Detail guess, max (',unitsymbol,'): ']; ...
        'Detail search settings: '});
    else
        switch analysissettings.isexcludeon
            case 0
                exlowstring = 'Disabled';
                exhighstring = 'Disabled';
            case 1
                exlowstring = [num2str(1/analysispar.excludehighfreq),' / ',num2str(analysissettings.excludelow)];
                exhighstring = [num2str(1/analysispar.excludelowfreq),' / ',num2str(analysissettings.excludehigh)];
        end
        switch analysissettings.isdetailauto
            case 0
                detlowstring = [num2str(1/analysispar.gaussianguess{2}),' / ',num2str(analysissettings.lowguess)];
                dethighstring = [num2str(1/analysispar.gaussianguess{1}),' / ',num2str(analysissettings.highguess)];
            case 1
                detlowstring = 'Auto';
                dethighstring = 'Auto';
        end
        switch analysissettings.isdetaildefault
            case 0
                detsettingstring1 = 'Custom';
                detsettingstring2 = ['[',num2str(analysissettings.segmentlength),'; ',num2str(analysissettings.alpha),'; ',num2str(analysissettings.angleslack),'; ',num2str(analysissettings.convergence_threshold),'; ',num2str(analysissettings.maxiterations),'; ',num2str(analysissettings.sigma_prior_angle),']'];
            case 1
                detsettingstring1 = 'Default';
                detsettingstring2 = ['[',num2str(analysissettings.default_segmentlength),'; ',num2str(analysissettings.default_alpha),'; ',num2str(analysissettings.default_angleslack),'; ',num2str(analysissettings.default_convergence_threshold),'; ',num2str(analysissettings.default_maxiterations),'; ',num2str(analysissettings.default_sigma_prior_angle),']'];
        end
        set(plothandle,'String', ...
        {['Excluded, min (',unitsymbol,'): ',exlowstring]; ...
        ['Excluded, max (',unitsymbol,'): ',exhighstring]; ...
        ['Detail guess, min (',unitsymbol,'): ',detlowstring]; ...
        ['Detail guess, max (',unitsymbol,'): ',dethighstring]; ...
        ['Detail search settings: ',detsettingstring1]; ...
        detsettingstring2});
    end
end
