function [background_polar2D] = estimateBackgroundPolar(PSD_polar, frequencies_polar, smoothingpercentage)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Form polar background PSD by averaging over all angles and take logarithms.
background_polar1D = mean(PSD_polar,1);
background_polar1D_log = log(background_polar1D);
frequencies_polar_log = log(frequencies_polar);

% Interpolate logarithmic spectrum to obtain equidistant points.
frequencies_polar_log_interp = linspace(min(frequencies_polar_log),max(frequencies_polar_log),length(frequencies_polar_log));
background_polar1D_log_interp = interp1(frequencies_polar_log,background_polar1D_log,frequencies_polar_log_interp,'linear');

% Perform smoothing via robust local regression using weighted linear least squares and a 1st degree polynomial model (rlowess)
% for the interpolated logarithmic spectrum.
smoothspectrum_interp = smooth(frequencies_polar_log_interp,background_polar1D_log_interp,smoothingpercentage,'rlowess');

% Interpolate the smoothed spectrum to get values at original logarithmic frequencies.
smoothspectrum = interp1(frequencies_polar_log_interp,smoothspectrum_interp,frequencies_polar_log,'linear');

% Convert the smoothed logarithmic spectrum to original linear scale to
% obtain a smooth 1D background spectrum.
background_polar1D_smooth = exp(smoothspectrum);

% Replicate the smooth 1D background spectrum to form a 2D background
% spectrum.
background_polar2D = repmat(background_polar1D_smooth',size(PSD_polar,1),1);
