function [thisorall, exportfiletype] = exportPrompt

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%% General components %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
screensize = get(0,'ScreenSize');
defaultBackground = get(0,'defaultUicontrolBackgroundColor');
exportwindow = figure('Name','Export settings','NumberTitle','Off','MenuBar','None', ...
    'WindowStyle','modal','CloseRequestFcn',@cancelbutton_Callback,'units','pixels','OuterPosition', ...
    [0.4*screensize(3) 0.3*screensize(4) 0.2*screensize(3) 0.4*screensize(4)],'Color',defaultBackground);
okbutton = uicontrol(exportwindow,'Style','pushbutton','String','Export','Callback',@okbutton_Callback,'units','normalized','position',[0.45 0.025 0.25 0.075]);
cancelbutton = uicontrol(exportwindow,'Style','pushbutton','String','Cancel','Callback',@cancelbutton_Callback,'units','normalized','position',[0.72 0.025 0.25 0.075]);

%%%%%%%%%%%%%%%%%%%%%% Export from %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
exportfromgroup = uibuttongroup('Parent',exportwindow,'Title','Export results from:','BorderType','none','units','normalized','position',[0.05 0.65 0.9 0.25]);
exportfrom = uicontrol(exportfromgroup,'Style','popupmenu','String','All analyzed images|This image only','BackgroundColor',[1 1 1],'units','normalized','position',[0.05 0.05 0.9 0.9]);

%%%%%%%%%%%%%%%%%%%%%% Export to %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
exporttypegroup = uibuttongroup('Parent',exportwindow,'Title','Export to:','BorderType','none','units','normalized','position',[0.05 0.4 0.9 0.25]);
exporttype = uicontrol(exporttypegroup,'Style','popupmenu','String','Spreadsheet summary|Text summary|Raw distributions (Warning: large file size!)|Figures','BackgroundColor',[1 1 1],'units','normalized','position',[0.05 0.05 0.9 0.9]);

%%%%%%%%%%%%%%%%%%%%%% User pressed ok, return selections %%%%%%%%%%%%%%%%%
function okbutton_Callback(hObject,eventdata)
    % Get this vs. all images selection.
    thisorall = get(exportfrom,'Value');
    switch thisorall
        case 1
            thisorall = 'All';
        case 2
            thisorall = 'This';
    end

    % Get filetype selection.
    exportfiletype = get(exporttype,'Value');
    switch exportfiletype
        case 1
            exportfiletype = 'spreadsheet';
        case 2
            exportfiletype = 'text';
        case 3
            exportfiletype = 'raw';
        case 4
            exportfiletype = 'figures';
    end
    delete(exportwindow);
end

%%%%%%%%%%%%%%%%%%%%% User pressed cancel, return empty %%%%%%%%%%%%%%%%%%%
function cancelbutton_Callback(hObject,eventdata)
    thisorall = [];
    exportfiletype = [];
    delete(exportwindow);
end

waitfor(exportwindow);
end