function [prior] = formGaussianPrior(gridsize, frequencies_polar, mu_angle, mu_freq, sigma_angle, sigma_freq)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Set up the parameters.
% To allow warping of the distribution at the 0/180 boundary, create copies
% of the grid and Gaussians.
priorsizey = 3*gridsize(1);
priorsizex = gridsize(2);
grid = [priorsizey priorsizex];
myy1=[priorsizey*(mu_angle/540),priorsizex*(mu_freq/max(frequencies_polar))];
myy2=[priorsizey*((mu_angle+180)/540),priorsizex*(mu_freq/max(frequencies_polar))];
myy3=[priorsizey*((mu_angle+360)/540),priorsizex*(mu_freq/max(frequencies_polar))];
sigma=[gridsize(2)*(sigma_freq/max(frequencies_polar)) gridsize(1)*(sigma_angle/180)];

% Form the Gaussians.
prior = zeros([grid 3]);
[prior(:,:,1)] = formGaussian2D(grid, myy1, sigma, 90);
[prior(:,:,2)] = formGaussian2D(grid, myy2, sigma, 90);
[prior(:,:,3)] = formGaussian2D(grid, myy3, sigma, 90);
prior = sum(prior,3);

% Get rid of the copies.
prior = prior(gridsize(1)+1:2*gridsize(1),:);
prior = prior/max(prior(:));